// File LIBBMP.H (used to build LIBBMP.A)
// Written by:
// Daniel W. Lewis (dlewis@scu.edu)
// Computer Engineering Department
// Santa Clara University
// Revised Jan 15, 2015

#include <stdio.h>
#include <stdint.h>
#include <stdarg.h>
#include <stdlib.h>

#include "bmp.h"

#define	MIN(a,b)	((a < b) ? a : b)
#define	MAX(a,b)	((a > b) ? a : b)

#pragma pack(1)     
typedef struct
	{
	uint16_t		type ;			// Magic identifier
	uint32_t		size ;			// File size in bytes
	uint16_t		reserved1 ;		// Not used
	uint16_t		reserved2 ;		// Not used
	uint32_t		offset ;			// Offset to image data in bytes
	uint32_t		header_size ;		// Header size in bytes
	uint32_t		width ;			// Width of the image
	uint32_t		height ;			// Height of image
	uint16_t		planes ;			// Number of color planes
	uint16_t		bits ;			// Bits per pixel
	uint32_t		compression ;		// Compression type
	uint32_t		imagesize ;		// Image size in bytes
	uint32_t		xresolution ;		// Pixels per meter
	uint32_t		yresolution ;		// Pixels per meter
	uint32_t		ncolours ;			// Number of colors  
	uint32_t		importantcolours ;	// Important colors
	} BMP_HEADER ;
#pragma pack()     

#define	INCHES_PER_METER	39.3701

#define PRIVATE static

void		_Error(char *format, ...) ;
unsigned	_BytesPerRow(unsigned cols) ;
void 		_AddPointer(IMAGE *image) ;
void 		_ChkPointer(IMAGE *image) ;
void 		_DelPointer(IMAGE *image) ;
RGB *		_GetPixel(IMAGE *image, unsigned row, unsigned col) ;
