// File READFILE.C (used to build LIBBMP.A)
// Written by:
// Daniel W. Lewis (dlewis@scu.edu)
// Computer Engineering Department
// Santa Clara University
// Revised Jan 15, 2015

#include "libbmp.h"

PRIVATE void _CheckFileFormat(char *filespec, BMP_HEADER *header) ;

IMAGE *ReadBMP24(char *filespec)
	{
	unsigned row_bytes ;
	BMP_HEADER header ;
	IMAGE *image ;
	FILE *fp ;
	int row ;

	if ((fp = fopen(filespec, "rb")) == NULL)
		_Error("Error opening file %s", filespec) ;

	if (fread(&header, sizeof(BMP_HEADER), 1, fp) != 1)
		_Error("Can't read BMP Header of file %s", filespec) ;

	_CheckFileFormat(filespec, &header) ;

	image = (IMAGE *) malloc(sizeof(IMAGE) + header.height * sizeof(RGB *)) ;
	if (image == NULL) _Error("Insufficient memory to load image %s", filespec) ;

	image->rows = header.height ;
	image->cols = header.width ;
	image->dpi  = header.xresolution / INCHES_PER_METER ;

	row_bytes = _BytesPerRow(header.width) ;
	for (row = 0; row < header.height; row++)
		{
		image->pxlrow[row] = (RGB *) malloc(row_bytes) ;
		if (image->pxlrow[row] == NULL) _Error("Insufficient memory to load image") ;
		if (fread(image->pxlrow[row], row_bytes, 1, fp) != 1)
			_Error("Can't read pixel data from file %s", filespec) ;
		}

	fclose(fp) ;

	_AddPointer(image) ;
	return image ;
	}

PRIVATE void _CheckFileFormat(char *filespec, BMP_HEADER *header)
	{
	if (header->type	 != 0x4D42)
		_Error("File %s contains incorrect BMP type (0x%04X)", filespec, header->type) ;

	if (header->bits	 != 24)
		_Error("File %s contains incorrect bits per pixel (%u)", filespec, header->bits) ;

	if (header->planes != 1)
		_Error("File %s contains incorrect number of planes (%u)", filespec, header->planes) ;

	if (header->compression != 0)
		_Error("File %s contains incorrect type of compression (%u)", filespec, header->compression) ;
	}
