// Library file WRITEFILE.C (used to build LIBWAV.A)
// Written by:
// Daniel W. Lewis (dlewis@scu.edu)
// Computer Engineering Department
// Santa Clara University
// Revised Jan 2, 2015

#include "libwav.h"

void WriteWAV16(char *filespec, AUDIO *audio)
	{
	unsigned sample_bytes ;
	unsigned sample_rate ;
	CHUNK_HDR chunk_hdr ;
	RIFF_INFO riff ;
	FMT_INFO fmt_info ;
	FILE *fp ;

	_ChkPointer(audio) ;

	sample_bytes = (unsigned) (audio->num_samples * sizeof(SAMPLE)) ;
	sample_rate  = (unsigned) audio->sample_rate ;

	if ((fp = fopen(filespec, "wb")) == NULL)
		_Error("Error creating file %s", filespec) ;

	chunk_hdr.id = 0x46464952 ; // 'RIFF'
	chunk_hdr.size	= sizeof(RIFF_INFO)
				+ sizeof(CHUNK_HDR) + sizeof(FMT_INFO)
				+ sizeof(CHUNK_HDR) + sample_bytes ;
	if (fwrite(&chunk_hdr, sizeof(CHUNK_HDR), 1, fp) != 1)
		_Error("Error writing RIFF chunk header of file %s", filespec) ;

	riff.format = 0x45564157 ; // 'WAVE'
	if (fwrite(&riff, sizeof(RIFF_INFO), 1, fp) != 1)
		_Error("Error writing RIFF format information of file %s", filespec) ;

	chunk_hdr.id = 0x20746D66 ; // 'fmt '
	chunk_hdr.size = 16 ;
	if (fwrite(&chunk_hdr, sizeof(CHUNK_HDR), 1, fp) != 1)
		_Error("Error writing format chunk header of file %s", filespec) ;

	fmt_info.format = 1 ; // PCM
	fmt_info.num_channels = 1 ;
	fmt_info.sample_rate = sample_rate ;
	fmt_info.byte_rate = sample_rate * sizeof(SAMPLE) ;
	fmt_info.block_align = sizeof(SAMPLE) ;
	fmt_info.bits_per_sample = 8 * sizeof(SAMPLE) ;
	if (fwrite(&fmt_info, sizeof(FMT_INFO), 1, fp) != 1)
		_Error("Error writing format information of file %s", filespec) ;

	chunk_hdr.id = 0x61746164 ;
	chunk_hdr.size = sample_bytes ;
	if (fwrite(&chunk_hdr, sizeof(CHUNK_HDR), 1, fp) != 1)
		_Error("Error writing DATA chunk header of file %s", filespec) ;

	if (fwrite(audio->samples, sizeof(SAMPLE), audio->num_samples, fp) != audio->num_samples)
		_Error("Error writing data to file %s", filespec) ;

	fclose(fp) ;
	}
